package cn.com.moonspeak.rabbitmq.monitor;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.io.IORuntimeException;
import cn.hutool.core.io.resource.Resource;
import cn.hutool.core.io.resource.ResourceUtil;
import cn.hutool.core.thread.ThreadUtil;
import cn.hutool.core.util.ArrayUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.extra.mail.MailAccount;
import cn.hutool.extra.mail.MailUtil;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import cn.hutool.json.JSONUtil;
import cn.hutool.setting.Setting;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * mq堆积监控
 * <p>
 * start.sh {host} {userName} {passWord} {mail}
 *
 * @author lisl
 * @version 1.0 - 2023/2/28
 * @class: MqQuery
 * @date 2023/2/28
 * @see
 */
public class Application {

    private static String baseUrl;
    private static String userName;
    private static String passWord;
    private static List<String> mails = new ArrayList<>();
    private static Integer thresholdCount;
    private static Boolean flag = false;
    private static final Logger log = LoggerFactory.getLogger(Application.class);
    private static Integer inNum = 0;
    private static Integer unNum = 0;

    @SneakyThrows
    public static void main(String[] args) {
//      初始化全局配置（具备自动更新能力）
        Setting setting = createDefaultSetting();
        baseUrl = setting.getStr("baseUrl", "http://120.26.55.29:15672/api");
        userName = setting.getStr("userName", "admin");
        passWord = setting.getStr("passWord", "admin");
        String mailArg = setting.getStr("mails", "853608143@qq.com");

        if (StrUtil.isNotBlank(mailArg)) {
            List<String> strings = StrUtil.splitTrim(mailArg, ",");
            if (CollUtil.isNotEmpty(strings)) {
                mails.addAll(strings);
            }
        }
        thresholdCount = Convert.toInt(setting.getStr("thresholdCount", "10"));
//      接口文档：http://120.26.55.29:15672/api/index.html#pagination
//      /queues 查询全部队列
//      /api/queues/(vhost)/(name)
        while (true) {
            String request = request("/queues?page=1&page_size=100&name=1_rates&use_regex=false&pagination=true");
            if (StrUtil.isBlank(request)) {
                sendMail("接口队列响应为空！可能出现异常，请检查！");
                continue;
            }
            MqQueueItemPageInfo result = JSONUtil.toBean(request, MqQueueItemPageInfo.class);
            MqQueueItemPageInfo.ItemsDTO itemsDTO = result.getItems().get(0);
            Integer itemCount = (itemsDTO.getMessagesReady() + itemsDTO.getMessagesUnacknowledged());
            log.info("{} {} {}",
                    itemsDTO.getMessagesReady(), itemsDTO.getMessagesUnacknowledged(),
                    itemCount);
            if ((itemCount > thresholdCount)) {
                // 超过阈值，清空正常情况计数
                unNum = 0;
                // 校验超过阈值计数
                if (inNum < 5) {
                    inNum++;
                } else {
                    // 超过阈值计数大于5进行预警通知
                    if (!flag) {
                        flag = true;
                        sendMail(StrUtil.format("接口队列处理堆积过大，目前为{},请处理！", itemCount));
                        log.info("接口队列处理堆积过大，目前为{},请处理！", itemCount);
                    }
                }
            } else {
                // 开始正常情况计数
                unNum++;
                // 正常情况计数大于5时清空超过阈值计数
                if (unNum > 5) {
                    inNum = 0;
                    flag = false;
                }
            }
            ThreadUtil.safeSleep(3000);
        }
    }

    public static final String[] GLOBAL_SETTING_PATHS = new String[]{"config/mq.setting", "file:mq.setting", "file:/config/mq.setting", "mq.setting"};

    public static Setting createDefaultSetting() {
        for (String settingPath : GLOBAL_SETTING_PATHS) {
            try {
                Setting setting = new Setting(settingPath);
                //在配置文件变更时自动加载
                setting.autoLoad(true);
                return setting;
            } catch (Exception ignore) {
                //ignore
//                ignore.printStackTrace();
            }
        }
        return null;
    }

    /**
     * 邮件批量发送
     *
     * @param content 邮件内容
     */
    public static void sendMail(String content) {
        try {
            MailUtil.send(mails, "酒店预警邮件", content, false, null);
        } catch (Exception e) {
            log.error("网络中断！目前为{}！", content);
        }
    }

    /**
     * 发送请求
     *
     * @param url
     * @return
     */
    public static String request(String url) {
        String json = null;
        try {
            HttpRequest httpRequest = HttpRequest.get(baseUrl + url);
            httpRequest.basicAuth(userName, passWord);
            HttpResponse response = httpRequest.execute();
            json = response.body();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return json;
    }

}
